@@ -1,24 +1,23 @@
 import os
-import json
-import imagehash
 import socket
-import urllib.request
-from PIL import Image, ImageDraw
+import imagehash
 from requests import get
+from PIL import Image, ImageDraw
+from json.decoder import JSONDecodeError
+from urllib.error import URLError
+from urllib.request import urlretrieve
 from requests.exceptions import ReadTimeout, ConnectTimeout
 from math import radians, degrees, sqrt, sin, asin, cos, atan2
 from WarThunder.maps import maps
 
 
-DEBUG = False
-
+LOCAL_PATH   = os.path.dirname(os.path.realpath(__file__))
+MAP_PATH     = os.path.join(LOCAL_PATH, 'map.jpg')
 IP_ADDRESS   = socket.gethostbyname(socket.gethostname())
 URL_MAP_IMG  = 'http://{}:8111/map.img'.format(IP_ADDRESS)
 URL_MAP_OBJ  = 'http://{}:8111/map_obj.json'.format(IP_ADDRESS)
 URL_MAP_INFO = 'http://{}:8111/map_info.json'.format(IP_ADDRESS)
-
-AIR_DIMENSION_KM = 65 # all air maps are 65kmx65km
-TEXTURES_PATH    = os.environ['APPDATA'] + r'\Tacview\Data\Terrain\Textures'
+MAX_HAMMING_DIST = 3
 EARTH_RADIUS_KM  = 6378.137
 REQUEST_TIMEOUT  = 0.1
 
@@ -71,13 +70,21 @@ def get_grid_info(map_img):
     '''
     
     hash_ = str(imagehash.average_hash(map_img))
+    hamming_dists = []
     
-    if hash_ in maps.keys():
-        return maps[hash_]
+    for map_hash in maps.keys():
+        hamming_dists.append((len([index for index in range(len(hash_)) if not hash_[index] == map_hash[index]]), map_hash))
+        
+        
+    match = min(hamming_dists)
+    
+    if match[0] <= MAX_HAMMING_DIST:
+        return maps[match[1]]
     
     return {'name': 'UNKNOWN',
             'ULHC_lat': 0.0,
-            'ULHC_lon': 0.0}
+            'ULHC_lon': 0.0,
+            'size_km' : 65}
 
 
 class MapInfo(object):
@@ -127,21 +134,21 @@ class MapInfo(object):
         self.map_valid = False
         
         try:
-            urllib.request.urlretrieve(URL_MAP_IMG, 'map.jpg')
+            urlretrieve(URL_MAP_IMG, MAP_PATH)
             self.info = get(URL_MAP_INFO, timeout=REQUEST_TIMEOUT).json()
             self.obj  = get(URL_MAP_OBJ,  timeout=REQUEST_TIMEOUT).json()
             
-            self.map_img  = Image.open('map.jpg')
+            self.map_img  = Image.open(MAP_PATH)
             self.map_draw = ImageDraw.Draw(self.map_img)
             
             self.grid_info = get_grid_info(self.map_img)
             
             self.map_valid = True
     
-        except urllib.error.URLError:
+        except URLError:
             print('ERROR: could not download map.jpg')
     
-        except json.decoder.JSONDecodeError:
+        except (OSError, JSONDecodeError):
             print('Waiting to join a match')
             
         except ReadTimeout:
@@ -149,9 +156,6 @@ class MapInfo(object):
             
         except ConnectTimeout:
             print('ERROR: ConnectTimeout')
-        
-        except FileNotFoundError:
-            print('ERROR: map.jpg not found')
             
         return self.map_valid
     
@@ -185,8 +189,8 @@ class MapInfo(object):
                         self.player_hdg += 360
                     
                     if self.grid_info:
-                        dist_x  = self.player_x * AIR_DIMENSION_KM
-                        dist_y  = self.player_y * AIR_DIMENSION_KM
+                        dist_x  = self.player_x * self.grid_info['size_km']
+                        dist_y  = self.player_y * self.grid_info['size_km']
                         dist    = hypotenuse(dist_x, dist_y)
                         bearing = degrees(atan2(self.player_y,
                                                 self.player_x)) + 90
